

c-----------------------------------------------------------------------------
c
c ROUTINE
c   Calc_Surf_Emiss 
c
c DESCRIPTION
c   This routine only calculates Surf_Emiss for use in expo. decay flag. 
c   It does not update const.
c
c ARGUMENTS
c   ai          : pressure = (ai * pt) + (bi * psf), ai at zone interface
c   bi          : pressure = (ai * pt) + (bi * psf), bi at zone interface
c   emiss_isop  : isoprene    emissions (kg/s)
c   emiss_monot : monoterpene emissions (kg/s)
c   emiss_nox   : NOx         emissions (kg/s)
c   mcor        : area of grid box  (m^2)
c   humidity    : specific humidity (g/kg)
c   pctm2       : CTM surface pressure at t1+tdt (mb)
c   kel         : temperature (degK)
c   emiss       : array of emissions (kg/s)
c   mass      : total mass of the atmosphere within each grid box (kg)
c   emiss       : array of emissions (kg/s)
c   surf_emiss  : surface  emissions (units?)
c   const       : species concentration, known at zone centers (mixing ratio)
c
c-----------------------------------------------------------------------------

      subroutine Calc_Surf_Emiss 
     &  (pr_surf_emiss, ai, bi, emiss_isop, emiss_monot, emiss_nox, 
     &   mcor,  humidity,
     &   pctm2, kel, emiss, surf_emiss)

      use h_imp_phys_constants
      use h_imp_time_constants
      use h_imp_dims
      use h_imp_control
      use h_imp_emiss_constants

      implicit none
 
c#     include "imp_phys_constants.h"
c#     include "imp_time_constants.h"
c#     include "imp_dims.h"
c#     include "imp_control.h"
c#     include "imp_emiss_constants.h" 

c     ----------------------
c     Argument declarations.
c     ----------------------

      logical :: pr_surf_emiss

      real*8  :: ai   (k1-1:k2)
      real*8  :: bi   (k1-1:k2)
      real*8  :: emiss_isop     (i1:i2, ju1:j2)
      real*8  :: emiss_monot    (i1:i2, ju1:j2)
      real*8  :: emiss_nox      (i1:i2, ju1:j2)
      real*8  :: mcor           (i1:i2, ju1:j2)
      real*8  :: surf_emiss_out (i1:i2, ju1:j2, num_species)
      real*8  :: humidity       (i1:i2, ju1:j2, k1:k2)
      real*8  :: pctm2(ilo:ihi, julo:jhi)
      real*8  :: kel  (ilo:ihi, julo:jhi, k1:k2)
c      real*8  :: mass (i1:i2, ju1:j2, k1:k2)
      real*8  :: emiss(i1:i2, ju1:j2, k1:k2, num_emiss, emiss_timpyr)
      real*8  :: surf_emiss (i1:i2, ju1:j2, num_species)
c      real*8  :: const      (i1:i2, ju1:j2, k1:k2, num_species)

c     -----------------------
c     Parameter declarations.
c     -----------------------

      real*8, parameter :: CMPM3 = CMPM * CMPM * CMPM

c     ----------------------------------------
c     Factors used for determining box_height.
c     ----------------------------------------

      real*8, parameter ::
     &  R29_3 = (GAS_CONST_J / MWTAIR * GPKG) / IMP_G,
     &  R_61  = (1.0d0 - (MWTH2O / MWTAIR)) / (MWTH2O / MWTAIR)


c     ----------------------
c     Variable declarations.
c     ----------------------

      integer :: ic, icx
      integer :: idumday, idumyear
      integer :: imon
      integer :: inum
      integer :: month

      real*8  :: box_height  (i1:i2, ju1:j2)
      real*8  :: conv_emiss  (i1:i2, ju1:j2)
      real*8  :: emiss_tscale(i1:i2, ju1:j2, k1:k1)
      real*8  :: mw_fac
c      real*8  :: emass(i1:i2, ju1:j2)

c     ----------------
c     Begin execution.
c     ----------------

      if (pr_diag) then
        Write (6,*) 'Calc_Surf_Emiss called by ', loc_proc
      end if

      if (emiss_timpyr == MONTHS_PER_YEAR) then
        call Split_Time_Flds (nymd, idumyear, month, idumday)
        imon = month
      else
        imon = 1
      end if

c     --------------------------------------------------------------
c     Calculate conversion factor to go from kg/box/s to mol/cm^3/s,
c     but leave out the molecular weight term for each species.
c     --------------------------------------------------------------

      box_height(:,:) = 
     &  R29_3 *
     &  (kel(i1:i2,ju1:j2,k1) *
     &   (1.0d0 + (R_61 * KGPG * humidity(i1:i2,ju1:j2,k1))) *
     &  Log (pctm2(i1:i2,ju1:j2) /
     &       ((ai(k1) * pt) + (bi(k1) * pctm2(i1:i2,ju1:j2)))))

      conv_emiss(:,:) =
     &  (AVOGAD * GPKG) / mcor(:,:) / CMPM3 / box_height(:,:)


c     -------------------------------------------------------------
c     Set surf_emiss; convert units from kg/s to ? for each species
c     that is emitted.
c     -------------------------------------------------------------

      inum = 0

      surf_emiss(:,:,:) = 0.0d0


c     ================================
      SPCLOOP: do icx = 1, num_species
c     ================================

        ic = emiss_map(icx)

        if (ic > 0) then

          inum = inum + 1

c          if (emiss_map_tfunc(icx) > 0 ) then

c            call Calc_Emiss_Tscale 
c     &        (emiss_map_tfunc(icx), k1, k1, emiss_tscale(:,:,k1))

c          else

            emiss_tscale(:,:,k1) = 1.0d0

c          end if

          surf_emiss(:,:,ic) =
     &      surf_emiss(:,:,ic) +
     &      emiss(:,:,k1,inum,imon) * emiss_tscale(:,:,k1) * 
     &      conv_emiss(:,:) / mw(ic)




c                                ============
          if (inum == num_emiss) exit SPCLOOP
c                                ============

        end if

c     ==============
      end do SPCLOOP
c     ==============


      if (emiss_opt == 2) then

c        write(6,*) ' gxlin-test2-------'
c       ==============================
        call Add_Semiss_Harvard_Inchem
c       ==============================
     &    (iisoprene_num, ico_num, ipropene_num, ino_num, mw,
     &     emiss_isop, emiss_monot, emiss_nox, conv_emiss, surf_emiss)

      end if



      return

      end


cc    -------------------------------------------------------------------
cc    Below is the original dust module provided by Paul Ginoux. It is 
cc    included here for the convenience of anyone analyzing our version.
cc    There should be no functional differences between the two versions.
cc    -------------------------------------------------------------------

c ****************************************************************************
c * This subroutine updates the surface mixing ratio of dust aerosols for
c * NMX size bins. The uplifting of dust depends in space on the source 
c * function, and in time and space on the soil moisture and surface
c * wind spedd (10 meters). Dust is uplifted if the wind speed is greater
c * than a threshold velocity which is calculated with the formula of
c * Marticorena et al.  (JGR, v.102, p 23277-23287, 1997).
c * To run this subroutine you need the source function which can be
c * obtained by contacting Paul Ginoux at ginoux@rondo.gsfc.nasa.gov
c * If you are not using GEOS DAS met fields, you will most likely need
c * to adapt the adjusting parameter.
c *
c * Contact: Paul Ginoux (ginoux@rondo.gsfc.nasa.gov) 
c *
c *
c *  Input:
c *         SRCE      Source function                               (-)
c *         DUSTDEN   Dust density                                  (kg/m3)
c *         DUSTREFF  Effective radius                              (um)
c *         DXY       Surface of each grid cell                     (m2)
c *         AIRMAS    Air mass for each grid box                    (kg)
c *         NTDT      Time step                                     (s)
c *         W10m      Velocity at the anemometer level (10meters)   (m/s)
c *         GWET      Surface wetness                               (-)
c *      
c *  Update:
c *         TC        Mixing ratio at first model level over Earth surface
c *
c *
c *  Parameters used in GOCART
c *
c *  Longitude: IMX = 144
c *  Latitude : JMX =  91
c *  Levels   : LMX =  20 (GEOS-1), 26 (GEOS-strat), 30 (GEOS-terra)
c *  Size bins: NMX =   4
c *
c *  Dust properties used in GOCART
c *
c *  Size classes: 01-1, 1-1.8, 1.8-3, 3-6 (um)
c *  Radius: 0.7, 1.5, 2.5, 4  (um)
c *  Density: 2500, 2650, 2650, 2650 (kg/m3)
c *
c *  References:
c *   1) Ginoux, P., M. Chin, I. Tegen, J. Prospero, B. Hoben, O. Dubovik,
c *      and S.-J. Lin, "Sources and distributions of dust aerosols simulated
c *      with the GOCART model", J. Geophys. Res., 2001
c *
c ****************************************************************************
c     
c     parameter (imx=144,jmx=91,lmx=26,nmx=4)
c Input
c     dimension GWET(imx,jmx),W10M(imx,jmx),SRCE(imx,jmx)
c     dimension AIRMAS(imx,jmx,LMX),DXY(imx,jmx)
c     dimension AIRDEN(imx,jmx,lmx)
c Mixing Ratio
c     dimension TC(imx,jmx,lmx,nmx)
c
c Soil size distribution 
c     dimension FRAC_S(nmx)
c
c     dimension DUSTREFF(nmx),DUSTDEN(nmx)
c Particle radii (m)
c     data DUSTREFF/0.75e-6,1.5e-6,2.5e-6,4e-6/
c Soil density (kg/m3)
c     data DUSTDEN/2500.,2650.,2650.,2650./
c
c Fraction of each size classes (deduced from Tegen and Fung, 1994)
c
c     data frac_s/0.1,0.25,0.25,0.25/
c
c     Ch_dust    = 1.e-9   ! Transfert coeff for type natural source 
c                            ! (kg*s2/m5)
c Loop over each size bins
c
c     do n = 1, nmx
c Threshold velocity as a function of the dust density and the diameter
c from Bagnold (1941), valid for particles larger than 10 um
c
c       u_ts0 = 6.5*sqrt(dustden(n)*g0*2.*dustreff(n))
c Threshold velocity from Greely and Iversen (1985)
c  Convert units to fit dimensional parameters
c       den=dustden(n)*1.e-3
c       diam=2.*dustreff(n)*1.e2
c       g=9.81*1.e2

c       TSRC = 0.
c
c Loop over each longitude
c
c       do i = 1, IMX
c
c Loop over each latitude
c
c         do j = 1, JMX
c           rhoa=airden(i,j,1)*1.e-3

c           u_ts0=0.13*1.e-2*sqrt(den*g*diam/rhoa)*
c    >        sqrt(1.+0.006/den/g/diam**2.5)/
c    >        sqrt(1.928*(1331*(diam)**1.56+0.38)**0.092-1)
c Case of surface dry enough to erode
c           if (gwet(i,j).lt.0.5) then
c             u_ts=amax1(0.,u_ts0*(1.2+0.2*alog10(amax1(1.e-3,
c    >           gwet(i,j)))))
c           else
c Case of wet surface, no erosion
c             u_ts=100.
c           end if
c           SRCE_p=frac_s(n)*SRCE(i,j)*DXY(i,j)  ! (m2)
c           DSRC=Ch_dust*SRCE_p*W10m(i,j)**2
c    >                * (W10m(i,j)-u_ts)*NTDT  ! (kg)
c           if (DSRC.lt.0.) DSRC=0.
c
c Update dust mixing ratio at first model level.
c
c           TC(i,j,1,n) = TC(i,j,1,n) + DSRC / AIRMAS(i,j,1)
c         end do
c       end do
c     end do
c    
c     RETURN
c     END

