
c-----------------------------------------------------------------------------
c
c ROUTINE
c   Add_Trop_Water
c
c DESCRIPTION
c   This routine adds the tropospheric water from the humidity variable into
c   the water vapor before doing chemistry; only do this below the calculated
c   tropopause pressure (tropp).  Make sure the water is never less than
c   2 ppm.
c
c ARGUMENTS
c   ih2o_num    : const array index for water
c   press3c     : atmospheric pressure at the center of each grid box (mb)
c   const       : species concentration, known at zone centers (mixing ratio)
c   tropp       : tropopause pressure (mb)
c   humidity    : specific humidity   (g/kg)
c   pchem_water : stratospheric and tropospheric water vapor before chemistry
c                 operator (mixing ratio)
c   strat_water : stratospheric water vapor before chemistry operator
c                 (mixing ratio)
c
c-----------------------------------------------------------------------------

        subroutine Add_Trop_Water
     &  (ih2o_num, press3c, const, tropp, humidity,
     &   pchem_water, strat_water)

      use h_imp_phys_constants
      use h_imp_dims
      implicit none

c#     include "imp_phys_constants.h"
c#     include "imp_dims.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: ih2o_num
      real*8  :: press3c(ilo:ihi, julo:jhi, k1:k2)
      real*8  :: const  (i1:i2, ju1:j2, k1:k2, num_species)
      real*8  :: tropp  (i1:i2, ju1:j2)
      real*8  :: humidity   (i1:i2, ju1:j2, k1:k2)
      real*8  :: pchem_water(i1:i2, ju1:j2, k1:k2)
      real*8  :: strat_water(i1:i2, ju1:j2, k1:k2)


c     -----------------------
c     Parameter declarations.
c     -----------------------

      real*8, parameter :: MIN_STRAT_WATER = 3.0d-6


c     ----------------------
c     Variable declarations.
c     ----------------------

      real*8  :: fac


c     ----------------
c     Begin execution.
c     ----------------

      if (pr_diag) then
        Write (6,*) 'Add_Trop_Water called by ', loc_proc
      end if


      strat_water(:,:,:) = const(:,:,:,ih2o_num)

        fac = MWTAIR / (MWTH2O * GPKG)


      where (press3c(i1:i2,ju1:j2,:) > Spread (tropp(:,:), 3, k2))

        const(:,:,:,ih2o_num) =
     &    const(:,:,:,ih2o_num) +
     &    (humidity(:,:,:) * fac)

      end where


      pchem_water(:,:,:) = const(:,:,:,ih2o_num)


      where (const(:,:,:,ih2o_num) < MIN_STRAT_WATER)
        const(:,:,:,ih2o_num) = MIN_STRAT_WATER
      end where

      if (pr_diag) then
        Write (6,*) 'Finish Add_Trop_Water called by ', loc_proc
      end if
   
      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Remove_Trop_Water
c
c DESCRIPTION
c   This routine removes the tropospheric water from the water vapor in th
c   const array, now that chemistry is finished.
c
c ARGUMENTS
c   ih2o_num    : const array index for water
c   const       : species concentration, known at zone centers (mixing ratio)
c   pchem_water : stratospheric and tropospheric water vapor before chemistry
c                 operator (mixing ratio)
c   strat_water : stratospheric water vapor before chemistry operator
c                 (mixing ratio)
c
c-----------------------------------------------------------------------------

      subroutine Remove_Trop_Water
     &  (ih2o_num, const, pchem_water, strat_water)

      use h_imp_dims
      implicit none

c#     include "imp_dims.h"

c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: ih2o_num
      real*8  :: const(i1:i2, ju1:j2, k1:k2, num_species)
      real*8  :: pchem_water(i1:i2, ju1:j2, k1:k2)
      real*8  :: strat_water(i1:i2, ju1:j2, k1:k2)


c     ----------------
c     Begin execution.
c     ----------------

      if (pr_diag) then
        Write (6,*) 'Remove_Trop_Water called by ', loc_proc
      end if

!!!!!!!!!Guangxing Lin add the protection from 0.
      where (pchem_water(:,:,:).eq.0.0)
        const(:,:,:,ih2o_num) = strat_water(:,:,:)
      
      elsewhere 

      const(:,:,:,ih2o_num) =
     &  strat_water(:,:,:) *
     &  (1.0d0 +
     &   ((const(:,:,:,ih2o_num) - pchem_water(:,:,:)) /
     &    pchem_water(:,:,:)))

      end where

      return

      end


