c=============================================================================
c
c $Id: quadinit.F,v 1.1 2003/05/16 22:22:25 akinorii Exp $
c
c CODE DEVELOPER
c   Original code from Sandy Sillman, U of M.
c   IMPACT quadchem modifications:  Akinori Ito.
c   2007 replacement:  Sandy Sillman, U of M
c
c FILE
c   quadinit.F
c
c ORIGINAL FILE
c   rbcheminit.F 2007
c
c=============================================================================

c ROUTINE:  quadinit  (also called cheminit)
c    cheminit (quadinit) :  Sets up variables relating to
c                chem. mechanism for solver
c                (settings that remain unchanged when solver is called)


c  cheminit.f   April, 2007
c
c  Program  written by Sanford Sillman
c  ---------------------------------------------------------
c  Program history:
c    12/06 Initial program by Sandy Sillman based on boxchemv7.f 
c  ---------------------------------------------------------
c     
c
          subroutine quadinit
c                   (cheminit)
c
c QUADINIT does a preliminary analysis of the chemical mechanism
c  to link reactions with individual species for the solution procedure
c  count odd hydrogen,
c  set up species pair chains for solution, etc.
c
c  Initial read for aqueous equilibrium was here, now in chemread
c
c  DETAILS:
c   1.  Each reaction is assigned to a species.
c        The numerical solution solves for each species in order
c        using the reactions needed for that species.
c
c   2.  Counts odd hydrogen and odd nitrogen - 
c                     net change for each  reaction.
c
c   3.  Identifies back-forth 'exchange' reaction numbers
c         associated with direct exchange of species
c         (e.g. special equilibria, MCO3-PAN, HNO4, etc - for excorr)
c
c   4.  Assigns reactions associated with 'cross-production' (cpro)
c        among paired species in 'cascade' solution (FEB 2005)
c
c   5.  Identifies 'pairfac': conservation factor for pair group species
c         based on back-forth exchange reactions (FEB 2005)
c         (e.g. Br+Br- <=>Br2- : Br2- counts double in pair group.)
c
c  FUTURE ADD:  Add category for soluble aerosol/aqueous species, 
c          and for aerosol odd-H.
c      Part of algorithm for non-steady state gas-aq partitioning
c       where initial gas/aqueous partitioning is set by input
c       for gas vs aqueous/aerosol species
c
c  FUTURE ADD:   Modify to skip aqueous reactions if LWC=0:
c      assign reactants to direct species, not gas pointer
c      and in chem, do reactions only when species is called 
c           - if LWC=0 skip aqueous
c
c       
c ****NOTE, TREATMENT OF AQUEOUS EQUILIBRIUM SPECIES NEEDS TO 
c     BE REWRITTEN AND CHANGED.
c 
c ---------------------------------------------
c Called by:  Do_Quad_Init (as part of chemical setup)
c              or by chemread
c   Called just once, as part of setup.
c Calls to:   none
c
c ---------------------------------------------
c History:
c  12/06 Written by Sandy Sillman from boxchemv7.f
c ---------------------------------------------------------------------
c      
      use h_rbchemmech
      use h_rbchemlocal
      IMPLICIT NONE

c#     include "rbchemmech.h"
c #     INCLUDE "rbchemvars.h"
c       Note:  miscellaneous variables from rbchemlocal.h used here.
c#     include "rbchemlocal.h"
c
c LOCAL VARIABLES
c lloss        Local: Flag for identifying exchange loss reaction
c lpro         Local: Flag for identifying exchange production reaction

c  xoddhx      Counter for net change in odd hydrogen in reaction
c                (intermediate for c_oddhx)
c  xpronox     Counter for net production of odd nitrogen in reaction
c                (intermediate for c_pronox)

      logical :: lloss   ! Flag for identifying exchange loss reaction
      logical ::  lpro   ! Flag for id exchange production reaction

      real*8 :: xoddhx      ! Counter for odd hydrogen RO2 only
      real*8 :: xoddhx2     ! Counter for odd hydrogen w/o RO2
      real*8 :: xpronox     ! Counter for odd nitrogen 
      integer :: nrp                  ! Counter for product reactions

c
c LOCAL DIMENSION
c         integer :: neqread(6)

c GENERAL FORMATS
11    format(///////,a1)
c 12    format(i6)
c 13    format((1pe10.3))

c
c -----------------------------------------
c  ANALYZE CHEMICAL MECHANISM:  
c   FOR EACH REACTION IDENTIFY THE (ONE) SPECIES THAT THE REACTION 
c   IS LINKED WITH IN THE SOLVER.  ALWAYS GAS-PHASE.
c    ALSO CALCULATE ODD HYDROGEN AND ODD NITROGEN CHANGE
c -----------------------------------------
c
c REACTIONS ARE ASSIGNED TO SPECIES IN REACTANT-TO-PRODUCT SEQUENCE
c WITH SPECIAL CATEGORIES (NO3, NOx, Hx) AT THE END.
c
c REACTIONS LINKED WITH A (GAS-PHASE) SPECIES INCLUDE:
c   (1) ALL SPECIES LOSS REACTIONS
c   (2) SPECIES PRODUCTION REACTIONS ONLY IF PRODUCTION IS FROM A 
c       "SPECIAL" CATEGORY THAT IS NOT INCLUDED 
c                    IN THE REACTION-TO-PRODUCT CASCADE.    
c   (3) REACTIONS FOR ASSOCIATED AQUEOUS SPECIES.
c   (4) REACTIONS FOR SPECIES LINKED THROUGH SPECIAL AQUEOUS EQUILIBRIA.

c THE IDENTIFICATION OF REACTIONS WITH SPECIES INSURES THAT 
c ALL REACTIONS WILL BE PROCESSED AT THE PROPER PLACE IN THE CASCADE.

c (NEW) LOSS AND PRODUCTION REACTIONS ARE ID'D SEPARATELY.
c -----------------------------------------

       kk=1


c  ZERO THE IMPORTANT ARRAYS
       do 160 ic=1,c_nchem2
        c_nnrchem(ic) = 0
        c_nnrchp(ic) = 0
        do 157 i=1,25
         c_nrchem(ic,i) = 0
         c_nrchmp(ic,i) = 0
157     continue
160    continue

       do 165 nr=1,c_nreac
         c_stoiloss(nr) = 0.
165    continue

c LOOP FOR EACH INDIVIDUAL REACTION
       do 1000 nr=1,c_nreac

c ZERO INDEX FOR IC ASSOCIATED WITH REACTION.
         ic = 0

c ESTABLISH REACTANT CATEGORIES. 
c     NOTE: REACTION IS ASSIGNED ONLY FOR ICAT>0.  CHECK ERROR.
c
       icat1 = 0
       icat2 = 0
       if(c_reactant(nr,1).gt.0) then
         icat1 = c_icat(c_reactant(nr,1))
       endif
       if(c_reactant(nr,2).gt.0) then
         icat2 = c_icat(c_reactant(nr,2))
       endif

c FIRST ELIMINATE REDUNDANT REACTANTS:  
c   REACTANTS THAT ARE ALSO PRODUCTS,  AND ALSO H2O
c
c  (This is for reactions RCO3+RO2=> products.
c    Entered as two reactions: 
c       RCO3+RO2=>RCO3+productB 
c       RCO3+RO2=>RO2 +productA 
c    Assign reaction with RO2 products to RO2, not RCO3.)

      if(c_nnpro(nr).gt.0) then
       do 305 n=1,c_nnpro(nr)
        if(c_reactant(nr,1).eq.c_product(nr,n).and.c_stoich(nr,n).ge.1) 
     *    icat1 = 0 - icat1
        if(c_reactant(nr,2).eq.c_product(nr,n).and.c_stoich(nr,n).ge.1) 
     *    icat2 = 0 - icat2
305    continue
      endif

c FIRST ASSOCIATE REACTION WITH REACTANTS
       if(icat1.lt.9.and.icat1.gt.0                ) then
         ic = c_reactant(nr,1)
       else
         if(icat2.lt.9.and.icat2.gt.0                ) then
           ic = c_reactant(nr,2)
         endif
       endif

c RESTORE CATEGORY OF REDUNDANT REACTIONS (RCO3+RO2)
       icat1 = abs(icat1)
       icat2 = abs(icat2)

c (PRIOR:  Look for NO3-N2O5 reactants before non-special products.
c  Now, Look for nonspecial products first, then NO3-N2O5 reactants.)


c IF NEITHER REACTANT IS A NON-SPECIAL SPECIES, THEN CHECK PRODUCTS
c  (e.g. HNO3, H2O2)
       if(ic.le.0) then
         if(c_nnpro(nr).gt.0) then
          do 310 n=1,c_nnpro(nr)
            icatp = c_icat(c_npequil(c_product(nr,n)))


            if(icatp.lt.9.and.icatp.gt.0                ) then
              ic = c_product(nr,n)
              go to 311
            endif
310       continue
311       continue
         endif
       endif

c IF NEITHER REACTANT IS A NON-SPECIAL SPECIES, LOOK FOR NO3-N2O5-HNO3.


       if(ic.le.0) then
         if(icat1.eq.14.or.icat1.eq.15.or.icat1.eq.16) then
           ic = c_reactant(nr,1)
         else
           if(icat2.eq.14.or.icat2.eq.15.or.icat2.eq.16) then
             ic = c_reactant(nr,2)
           endif
         endif
       endif


c IF NOTHING, LOOK FOR NO3/N2O5/HNO3 IN PRODUCTS
       if(ic.le.0) then
            if(c_nnpro(nr).gt.0) then
             do 320 n=1,c_nnpro(nr)
              icatp = c_icat(c_npequil(c_product(nr,n)))
              if(icatp.eq.14.or.icatp.eq.15.or.icatp.eq.16) then
                ic = c_product(nr,n)
                go to 321
              endif
320          continue
321          continue
            endif
       endif

c IF STILL NOTHING, LOOK FOR O3 or NOx IN REACTANTS OR PRODUCTS
c    O3 (11), NO2 (12) or NO (13)

c 2005 CHANGE   O3, NO, NO2 ORDER DOESN'T MATTER, 
c   BUT DECLARE THROUGH REACTANTS NOT PRODUCTS - else brpro too early
c   (HO2+NO=>NO2+OH must  be assigned to NO2)
c   MUST BE  11 or 12 or 13 here

       if(ic.le.0) then
c        if(icat1.eq.11.or.icat1.eq.12) then
         if(icat1.eq.11.or.icat1.eq.12.or.icat1.eq.13) then
           ic = c_reactant(nr,1)
         else
c          if(icat2.eq.11.or.icat2.eq.12) then
           if(icat2.eq.11.or.icat2.eq.12.or.icat2.eq.13) then
             ic = c_reactant(nr,2)
           else
            if(c_nnpro(nr).gt.0) then
             do 330 n=1,c_nnpro(nr)
              icatp = c_icat(c_npequil(c_product(nr,n)))
c             if(icatp.eq.11.or.icatp.eq.12) then
              if(icatp.eq.11.or.icatp.eq.12.or.icatp.eq.13) then
                ic = c_product(nr,n)
                go to 331
              endif
330          continue
331          continue
            endif
           endif
         endif
       endif
c
c H2O2 OPTION:  MAKE H2O2 A SPECIAL SPECIES FOR ASSIGNING REACTIONS:
c LOOK FOR H2O2 (19 ) IN REACTANTS OR PRODUCTS.  
       if(ic.le.0) then
         if(               icat1.eq.19) then
           ic = c_reactant(nr,1)
         else
           if(               icat2.eq.19) then
             ic = c_reactant(nr,2)
           else
            if(c_nnpro(nr).gt.0) then
             do 345 n=1,c_nnpro(nr)
              icatp = c_icat(c_npequil(c_product(nr,n)))
              if(               icatp.eq.19) then
                ic = c_product(nr,n)
                go to 346
              endif
345          continue
346          continue
            endif
           endif
         endif
       endif

c IF STILL NOTHING, THE REMAINDER HAD BETTER HAVE OH OR HO2!
c  (MAYBE TRY TO USE OH RATHER THAN HO2?)
       if(ic.le.0) then
         if(icat1.eq.9.or.icat1.eq.10) then
           ic = c_reactant(nr,1)
         else
           if(icat2.eq.9.or.icat2.eq.10) then
             ic = c_reactant(nr,2)
           else
            if(c_nnpro(nr).gt.0) then
             do 350 n=1,c_nnpro(nr)
              icatp = c_icat(c_npequil(c_product(nr,n)))
              if(icatp.eq.9.or.icatp.eq.10) then
                ic = c_product(nr,n)
                go to 351
              endif
350          continue
351          continue
            endif
           endif
         endif
       endif

c
c IF STILL IC=0, ERROR AND EXIT!! (unless reaction=0)
      if(ic.le.0.and.
     *     (c_reactant(nr,1).gt.0.or.c_reactant(nr,2).gt.0)) then
      write(c_out,355) nr, c_reactant(nr,1), icat1,
     *                  c_reactant(nr,2), icat2,
     *  (c_product(nr,n),n=1,c_nnpro(nr) )
      write(6,355) nr, c_reactant(nr,1), icat1,
     *                  c_reactant(nr,2), icat2,
     *  (c_product(nr,n),n=1,c_nnpro(nr) )
355   format(//,'MAJOR ERROR:  REACTION NOT IDENTIFIED WITH SPECIES',
     * //,' REACTION NUMBER =',i4,/,
     *    ' FIRST REACTANT =', i4,'  CATEGORY =',i3,/,
     *    ' SECOND REACTANT=', i4,'  CATEGORY =',i3,/,
     *    ' PRODUCTS =', (10i5)  )
3000   stop 3000
      endif
c

c FINALLY!  ENTER REACTION INTO PROPER SPECIES LIST
c NEW - SEPARATE LIST FOR SPECIES-LOSS REACTIONS AND PRODUCT REACTIONS,
c       AND CALC OF STOILOSS FOR LOSS REACTIONS.
c 2000-REACTIONS ASSIGNED TO MAIN SPECIES FOR SPECIAL AQUEOUS LINKS.

c  FUTURE ADD:   Modify to skip aqueous reactions if LWC=0:
c      assign reactants to direct species, not gas pointer
c      and in chem, do reactions only when species is called 
c           - if LWC=0 skip aqueous
c

      if(ic.gt.0) then
        icc = c_npequil(ic)
        if(ic.eq.c_reactant(nr,1).or.ic.eq.c_reactant(nr,2)) then
          c_nnrchem(icc) = c_nnrchem(icc) + 1
          c_nrchem(icc,c_nnrchem(icc)) = nr
          c_nicreac(nr) = icc
          c_stoiloss(nr) = 1.
          if(c_reactant(nr,1).eq.c_reactant(nr,2)) c_stoiloss(nr) = 2.
          if(c_nnpro(nr).gt.0) then
            do 360 n=1,c_nnpro(nr)
             if(ic.eq.c_product(nr,n)) 
     *          c_stoiloss(nr) = c_stoiloss(nr)-c_stoich(nr,n)
360         continue
          endif
        else
          c_nnrchp(icc) = c_nnrchp(icc) + 1
          c_nrchmp(icc,c_nnrchp(icc)) = nr
        endif
      endif

c END ANALYSIS OF SPECIES-LINK WITH REACTIONS.


c  CALCULATE CHANGE IN ODD HYDROGEN (ODDHX)
c  AND NET PRODUCTION OF ODD NITROGEN (PRONOX, >0).
c  (CURRENT OPTION - RECORD NET PRONOX, THEN IN PROGRAM 
c   SUM NOx PRODUCTION-ONLY.)
c  (ODDHX IS NET CHANGE, +/-.  BUT PRONOX IS PRODUCTION ONLY.
c   NET CHANGE IN NOX CAN BE OBTAINED FROM RP, RL FOR NO AND NO2,
c   BUT NOX ANALYSIS REQUIRES SEPARATION OF NOX SOURCES AND SINKS.
c   NOTE - PAN->NOX AND HNO3->NOX OVERPRODUCTION IS FIXED 
c   BY PANCORR AND NO3CORR - DISCOUNTS BACK-FORTH NOX PRODUCTION.)

      xoddhx = 0.
      xoddhx2 = 0.
      xpronox = 0.


      if(icat1.eq.9.or.icat1.eq.10.or.icat1.eq.8)
     &                                          xoddhx2 = xoddhx2 - 1.
      if(icat2.eq.9.or.icat2.eq.10.or.icat1.eq.8)
     &                                          xoddhx2 = xoddhx2 - 1.
      if(icat1.eq.3)
     &                                            xoddhx = xoddhx - 1.
      if(icat2.eq.3)
     &                                            xoddhx = xoddhx - 1.

      if(icat1.eq.12.or.icat1.eq.13) xpronox = xpronox - 1.
      if(icat2.eq.12.or.icat2.eq.13) xpronox = xpronox - 1.

c TEMPORARY TEST
c     if(icat1.eq.3.and.icat2.eq.3) 
c    *     write(c_out,12001) nr, (c_treac(ir,nr),ir=1,5),
c    *         xoddhx

          if(c_nnpro(nr).gt.0) then
      do 410 n=1,c_nnpro(nr)
       icatp = c_icat(c_npequil(c_product(nr,n)))

       if(icatp.eq.9.or.icatp.eq.10.or.icatp.eq.8)
     *     xoddhx2 = xoddhx2 + c_stoich(nr,n)
       if(icatp.eq.3)
     *     xoddhx = xoddhx + c_stoich(nr,n)

       if(icatp.eq.12.or.icatp.eq.13)
     *     xpronox = xpronox + c_stoich(nr,n)


c TEMPORARY TEST
c     if(icat1.eq.3.and.icat2.eq.3) write(c_out,*) icatp
c     if(icat1.eq.3.and.icat2.eq.3) 
c    *     write(c_out,12001) nr, (c_treac(ir,nr),ir=1,5),
c    *         xoddhx
c 12001      format(i5,2x,a8,'+',a8,'=>',a8,'+',a8,'+',a8,/,3(1pe10.3))


410   continue
           endif

       c_oddhx(nr,1) = xoddhx + xoddhx2
       c_oddhx(nr,2) = xoddhx2
       c_pronox(nr) = xpronox

c DEBUG -  TEST WRITE  
c 92407 TEMPORARY  
       if(c_kkw.gt.0) then
c        if(nr.gt.280.and.nr.le.284) then
          write(c_out,1001) nr,(c_treac(j,nr),j=1,5),ic,c_tchem(ic)
     *    , c_oddhx(nr,1),c_oddhx(nr,2), c_pronox(nr)
1001    format('REACTION #',i4,': ',a8,'+',a8,'=>',a8,',',a8,',',a8,/,
     *     ' KEY SPECIES =',i4,2x,a8,'  ODD-H=',2f8.2,'  PRONOX=',f8.2)
          write(c_out,1002) icc, c_nicreac(nr), c_nnrchem(icc)
     *                      , c_nnrchp(icc)
1002      format('ICC,NICREAC, NNRCHEM NNRCHMP=',4i5)
          if(c_nnrchem(icc).gt.0) write(c_out,1003)
     *               c_nrchem(icc,c_nnrchem(icc)),c_stoiloss(nr)
1003      format('NRCHEM, STOILOSS=',i5,f8.2)
          if(c_nnrchp(icc).gt.0) 
     *       write(c_out,1004) c_nrchmp(icc,c_nnrchp(icc))
1004      format('NRCHEMP=',2i5)
        endif

c ADDITION FOR ODD HYDROGEN:  IDENTIFY THE HO2->OH DIRECT REACTIONS.
c
c        if( (c_npequil(c_reactant(nr,1)).eq.c_nho2     .or.
c    *        c_npequil(c_reactant(nr,2)).eq.c_nho2     ).and.
c    *        c_nnpro(nr).gt.0)    then
c
         if(c_nnpro(nr).gt.0)      then
                if(c_npequil(c_reactant(nr,1)).ne.c_nho2     ) then
                  if(c_reactant(nr,2).le.0) then
                    go to 431
                  else
                    if(c_reactant(nr,2).ne.c_nho2     ) go to 431
                  endif
                endif
          do 430 n=1,c_nnpro(nr)
            if( c_npequil(c_product(nr,n)).eq.c_noh      ) then
              go to 431
            endif
430       continue
431       continue

         endif

1000    continue
c END REACTION LOOP - END MECHANISM ANALYSIS.


c ----------------------------------------------------------
c IDENTIFY 'EXCHANGE' REACTIONS WITH THE FORM A+B=>C; C=>A+B.
c AND PAIRFAC/MULTIFAC CONSERVATION INDICES.
c
c THESE REACTION NUMBERS ARE RECORDED IN EXPRO(IC,I) AND EXLOSS(IC,I)
c WHERE IC IS FOR THE PRODUCT C  (GAS-PHASE).

c THE EXCHANGE COEFFICIENTS INVOKE THE SUBROUTINE HNCORR (w/PANCORR)
c TO INSURE THAT THE BACK-AND-FORTH PRODUCTION AND LOSS
c DO NOT INTERFERE WITH BACK-EULER SOLUTION FOR PRODUCT SPECIES
c  (USUALLY NOX OR HX).
c THIS IS ALSO IMPORTANT FOR AQUEOUS:  HCO3+O2- ->HO2- + CO3-.
c
c NOTE:  THIS ONLY WORKS IF EXCHANGE REACTIONS HAVE THE SAME KEY SPECIES
c OR IF THE TWO KEY SPECIES ARE DONE WITH TWOSOLVE (RCO3-PAN).
c
c NOTE:  EXLOSS AND EXPRO ARE ENTERED AS PAIRED REACTIONS;  
c  BUT SAME EXPRO MAY GO TO MORE THAN ONE EXLOSS, OR  VICE VERSUS.
c  (so same reaction may be listed twice)

c ----------------------------------------------------------
c  ZERO THE IMPORTANT ARRAYS
       do  ic=1,c_nchem2
        do i=1,20
         c_exspec(ic,i) = 0
         do ii=1,5
           c_expro(ic,i,ii) = 0 
           c_exloss(ic,i,ii) = 0
         enddo               !do ii=1,5
        enddo             !do i=1,20
       enddo            !do  ic=1,c_nchem2

c INITIALIZE PAIRFAC (conservation parameter for pair species)
c   = 1 unless adjusted by EXCHANGE REACTIONS

       do ic=1,c_nchem2
         c_pairfac(ic) = 1.
         c_multfac(ic) = 1.
       enddo             !do ic=1,c_nchem2
c
c --------
c BEGIN REACTION LOOP FOR EXCHANGE REACTIONS (1200)

       do nrx=1,c_nreac

c EXLOSS REACTION:  
c  ICX (NICREAC) is key species for reaction (1st non-special reactant).
          icx = c_nicreac(nrx)

c  Identify up to two REACTANTS ICY1, ICY2; 
c               and up to two PRODUCTS ICX1,ICX2.

          icy1 = c_npequil(c_reactant(nrx,1))
          icy2 = 0
          if (c_reactant(nrx,2).gt.0) then
            icy2 = c_npequil(c_reactant(nrx,2))
          endif

          icx1 = 0
          icx2 = 0
          if(c_product(nrx,1).gt.0) icx1=c_npequil(c_product(nrx,1))
          if(c_product(nrx,2).gt.0) icx2=c_npequil(c_product(nrx,2))

          icat1 = 0
          icat2 = 0
          icatp = 0
          if(icy1.gt.0) then
             icat1 = c_icat(icy1)
          endif
          if(icy2.gt.0) then
             icat2 = c_icat(icy2)
          endif
          if(icx1.gt.0) then
             icatp = c_icat(icx1)
          endif


c CONTROLS TO SKIP LOOP (1200):  
c
c SKIP AND CONTINUE LOOP IF (a) TWO REACTANTS, ONE PRODUCT; 
c  OR IF (b) KEY SPECIES IS A PRODUCT SPECIES.  (IN WHICH CASE
c  THIS REACTION WOULD BE THE EXCHANGE 'RETURN' - EXPRO, not EXLOSS)
          if(icy2.gt.0.and.icx2.eq.0) then
          else
           if(icx.ne.icy1.and.icx.ne.icy2.and.icy2.gt.0) then
           else

c OPTIONAL CRITERIA:  EXCHANGE FOR FAST SPECIES ONLY.  FOR SLOW SPECIES,
c SKIP AND CONTINUE LOOP
            if(c_icat(icy1).eq.1.or.c_icat(icy1).eq.4) then
            else

c SEARCH THROUGH REACTIONS TO FIND MATCHING EXPRO REACTION(s) (1300)
              do nrp=1,c_nreac

c  ICP (NICREAC) is key species for EXPRO reaction 
               icp = c_nicreac(nrp)

c  Identify EXPRO reactants IC1,  IC2 and products ICP1, ICP2.
               ic1=0
               ic2=0
               if(c_reactant(nrp,1).gt.0) then
                  ic1=c_npequil(c_reactant(nrp,1))
               endif
               if(c_reactant(nrp,2).gt.0) then
                  ic2=c_npequil(c_reactant(nrp,2))
               endif
               icp1=0
               icp2=0
               if(c_product(nrp,1).gt.0) then
                  icp1=c_npequil(c_product(nrp,1))
               endif
               if(c_product(nrp,2).gt.0) then
                  icp2=c_npequil(c_product(nrp,2))
               endif

c CONTROLS TO IDENTIFY  EXCHANGE REACTION (EXLOSS-EXPRO):  A+B=C, C=A+B
               if( (icy1.eq.icp1.and.icy2.eq.icp2).or.
     *                  (icy1.eq.icp2.and.icy2.eq.icp1) )  then
                 if( (icx1.eq.ic1.and.icx2.eq.ic2).or.
     *                  (icx1.eq.ic2.and.icx2.eq.ic1) )  then

c SET PAIRFAC (pair group conservation of mass) 
c   FOR EXCHANGE REACTIONS A+B=C
c    (note A+B=>C must be return reaction)

                 if(ic1.ne.0.and.ic2.ne.0.and.icp1.ne.0) then 
                   if(c_nppair(ic1,2).eq.c_nppair(ic2,2)) then
                     if(c_nppair(icp1,2).eq.c_nppair(ic1,2)) then
                       if(icp2.eq.0) then
                          c_pairfac(icp1) = 
     *                       c_pairfac(ic1)+c_pairfac(ic2)
c TEST WRITE - QUADINIT PAIRFAC (STANDARD)
                          if(c_kkw.gt.0) write(c_out,1209) 
     *                     c_tchem(ic1), c_tchem(ic2), c_tchem(icp1),
     *                     c_pairfac(ic1), c_pairfac(ic2), 
     *                     c_pairfac(icp1)
1209                       format(' SET PAIRFAC:  species A + B <->C.',
     *                            ' SPECIES AND PAIRFACS = ',/,
     *                            3(a8,2x), 3(1pe10.3)                )
                       else
                          if(c_nppair(icp2,2).ne.c_nppair(ic1,2)) then
                            c_pairfac(icp2) = 
     *                         c_pairfac(ic1)+c_pairfac(ic2)
c TEST WRITE - QUADINIT PAIRFAC (STANDARD)
                            if(c_kkw.gt.0) write(c_out,1209) 
     *                       c_tchem(ic1), c_tchem(ic2), c_tchem(icp1)
                          endif
                       endif               !if(icp2.eq.0) then
                     else         !if(c_nppair(icp1,2).eq....
                       if(icp2.gt.0) then
                          if(c_nppair(icp2,2).eq.c_nppair(ic1,2)) then
                            c_pairfac(icp2) = 
     *                         c_pairfac(ic1)+c_pairfac(ic2)
c TEST WRITE - QUADINIT PAIRFAC (STANDARD)
                            if(c_kkw.gt.0) write(c_out,1209) 
     *                       c_tchem(ic1), c_tchem(ic2), c_tchem(icp1)
                          endif
                       endif               !if(icp2.gt.0) then
                     endif           !if(c_nppair(icp1,2).eq....
                   endif          !if(c_nppair(ic1,2).eq....
                 endif        !if(ic1.ne.0.and.ic2.ne.0.and.icp1.ne.0)..

c
c SET MULTIFAC (multi group conservation of mass) 
c   FOR EXCHANGE REACTIONS A+B=C
c    (note A+B=>C must be return reaction)

                 if(ic1.ne.0.and.ic2.ne.0.and.icp1.ne.0) then 
                   if(c_npmulti(ic1,1).eq.c_npmulti(ic2,1)) then
                     if(c_npmulti(icp1,1).eq.c_npmulti(ic1,1)) then
                       if(icp2.eq.0) then
                          c_multfac(icp1) = 
     *                       c_multfac(ic1)+c_multfac(ic2)
c TEST WRITE - QUADINIT MULTIFAC (STANDARD)
                          if(c_kkw.gt.0) write(c_out,1208) 
     *                     c_tchem(ic1), c_tchem(ic2), c_tchem(icp1),
     *                     c_multfac(ic1), c_multfac(ic2), 
     *                               c_multfac(icp1)
1208                       format(' SET MULTIFAC:  species A + B <->C.',
     *                            ' SPECIES AND MULTIFACS = ',/,
     *                            3(a8,2x), 3(1pe10.3)                )
                       else
                          if(c_npmulti(icp2,1).ne.c_npmulti(ic1,1)) then
                           c_multfac(icp2) = 
     *                          c_multfac(ic1)+c_multfac(ic2)
c TEST WRITE - QUADINIT MULTIFAC (STANDARD)
                            if(c_kkw.gt.0) write(c_out,1208) 
     *                       c_tchem(ic1), c_tchem(ic2), c_tchem(icp1)
                          endif
                       endif               !if(icp2.eq.0) then
                     else            !if(c_npmulti(icp1,1).eq....
                       if(icp2.gt.0) then
                          if(c_npmulti(icp2,1).eq.c_npmulti(ic1,1)) then
                           c_multfac(icp2) = 
     *                        c_multfac(ic1)+c_multfac(ic2)
c TEST WRITE - QUADINIT MULTIFAC (STANDARD)
                            if(c_kkw.gt.0) write(c_out,1208) 
     *                       c_tchem(ic1), c_tchem(ic2), c_tchem(icp1)
                          endif
                       endif               !if(icp2.gt.0) then
                     endif           !if(c_npmulti(icp1,1).eq....
                   endif          !if(c_npmulti(ic1,1).eq....
                 endif        !if(ic1.ne.0.and.ic2.ne.0.and.icp1.ne.0)

c SET LOGIC FLAGS TO TEST IF REACTION IS ALREADY LISTED
                   lloss = .TRUE.
                   lpro  = .TRUE.

c LOOP TO TEST WHETHER EXCHANGE REACTION IS ALREADY LISTED IN REVERSE
c   If listed, EXPRO reaction would be  EXLOSS for its key species

                   if(icp.gt.0) then
                    do i=1,20
                     do ii=1,5
c IF EXCHANGE RN IS LISTED IN REVERSE, USE LPRO, LLOSS to ID reaction as listed.
                      if(c_exloss(icp,i,ii).eq.nrp) then
                       lloss= .FALSE.
                       lpro= .FALSE.
                      endif
                     enddo          !do ii=1,5
                    enddo        !do i=1,20
                   endif       !if(icp.gt.0) 

c LOOP TO TEST WHETHER EXLOSS, PRO REACTIONS ARE ALREADY IN FORWARD LIST

                   do i=1,20
                    do ii=1,5
c IF EXCHANGE RN IS LISTED IN REVERSE, USE LPRO, LLOSS to ID reaction as listed.
                     if(c_exloss(icx,i,ii).eq.nrx) lloss = .FALSE.
                     if( c_expro(icx,i,ii).eq.nrp)  lpro = .FALSE.
                    enddo          !do ii=1,5
                   enddo        !do i=1,20

c IDENTIFY EXCHANGED SPECIES (icp):  
c  Record species only if base and exchanged species 
c                                         are in same pair group.
c   and if exchanged  species is in reaction product list
c  If no exchanged species, flag  species number as -1.
                   if(icp.gt.0) then
                    if( icp.ne.icx1.and.icp.ne.icx2)   icp=0
                    if(c_nppair(icx,2).ne.c_nppair(icp,2)) icp=0
                   endif           !if(icp.gt.0) 
                   if(icp.eq.0)  icp=-1

c RECORD EXCHANGE SPECIES ONLY IF MAIN AND EXCHANGE SPECIES ARE PAIRED 
c  AND IF KEY SPECIES OF 2ND REACTION IS PRODUCT OF 1ST 
c
c  ( EXCORR must be called for single species also, e.g. HNO4.

c LOOP TO ADD EXCHANGED SPECIES TO EXSPEC LIST

                  if(lloss.or.lpro) then
                    do i=1,20
                     is=i
                     if(c_exspec(icx,i).eq.icp) go to 1205
                     if(c_exspec(icx,i).eq.0) then
                       c_exspec(icx,i)  = icp
                       go to 1205
                     endif
                     if(i.eq.20) write(c_out,1206)
                     if(i.eq.20) write(  6 ,1206)
1206                 format(/,' MAJOR ERROR IN QUADINIT: ',
     *                'EXCHANGED SPECIES INDEX EXCEEDED LIMIT' ,
     *                ' (i=1,20, 1205)',/)
                    enddo           !   do i=1,20
1205                continue
                  endif          !if(lloss.or.lpro) 

c LOOP TO ADD EXLOSS REACTION
                  if(lloss) then
                   do ii=1,5
c ENTER THE EXCHANGE REACTION
                    if(c_exloss(icx,is,ii).eq.0) then
                      c_exloss(icx,is,ii)=nrx
c TEST WRITE - QUADINIT EXCHANGE(STANDARD)
c 92407 TEMPORARY
                     if(c_kkw.gt.0) 
c                      if(c_kkw.ge.0) 
     *                   write(c_out,1212) icx,icp, is,i,nrx,
     *                   c_reactant(nrx,1),c_reactant(nrx,2),
     *                   c_product(nrx,1),c_product(nrx,2)
1212                     format (/,'EXLOSS(ICX,IS,I)=NRX. ',  
     *                   'ICX,ICP,IS,I,NRX=', 5i5,/,
     *                   ' REACTANTS=',2i5, '  PRODUCTS=',2i5   )
                      if(c_kkw.gt.0.and.c_reactant(nrx,1).gt.0)
     *                 write(c_out,*) c_tchem(c_reactant(nrx,1))
                      if(c_kkw.gt.0.and.c_reactant(nrx,2).gt.0)
     *                 write(c_out,*) c_tchem(c_reactant(nrx,2))
                      if(c_kkw.gt.0.and.c_product(nrx,1).gt.0)
     *                 write(c_out,*) c_tchem( c_product(nrx,1))
                      if(c_kkw.gt.0.and. c_product(nrx,2).gt.0)
     *                 write(c_out,*) c_tchem( c_product(nrx,2))

c HAVING ENTERED THE REACTION, BREAK THE LOOP
                      go to 1211
c ERROR WRITE
                      write(c_out,1213)
                      write( 6  ,1213)
1213                 format(/,' MAJOR ERROR IN QUADINIT: ',
     *                'EXLOSS INDEX EXCEEDED LIMIT' ,
     *                ' (ii=1,5, 1213)',/)

                    endif           ! if(c_exloss(icx,is,ii).eq.0
                   enddo      !do ii=1,5 (1210)
                  endif         !if(lloss) 
c END LOOP TO ADD EXLOSS REACTION (1210)
c    CONTINUE AFTER BREAK LOOP
1211              continue
c

c LOOP TO ADD EXPRO  REACTION
                  if(lpro ) then
                   do ii=1,5
c ENTER THE EXCHANGE REACTION
                    if( c_expro(icx,is,ii).eq.0) then
                       c_expro(icx,is,ii)=nrp
c TEST WRITE - QUADINIT EXCHANGE(STANDARD)
c 92407 TEMPORARY
                     if(c_kkw.gt.0) 
c                      if(c_kkw.ge.0) 
     *                   write(c_out,1222) icx,icp, is,i,nrp,
     *                   c_reactant(nrp,1),c_reactant(nrp,2),
     *                   c_product(nrp,1),c_product(nrp,2)
1222                     format (/,' EXPRO(ICX,IS,I)=NRX.  ',
     *                   'ICX,ICP,IS,I,NRX=', 5i5,/,
     *                   ' REACTANTS=',2i5,'  PRODUCTS=',2i5   )
                      if(c_kkw.gt.0.and.c_reactant(nrp,1).gt.0)
     *                 write(c_out,*) c_tchem(c_reactant(nrp,1))
                      if(c_kkw.gt.0.and.c_reactant(nrp,2).gt.0)
     *                 write(c_out,*) c_tchem(c_reactant(nrp,2))
                      if(c_kkw.gt.0.and.c_product(nrp,1).gt.0)
     *                 write(c_out,*) c_tchem( c_product(nrp,1))
                      if(c_kkw.gt.0.and. c_product(nrp,2).gt.0)
     *                 write(c_out,*) c_tchem( c_product(nrp,2))

c HAVING ENTERED THE REACTION, BREAK THE LOOP
                      go to 1221
c ERROR WRITE
                      write(c_out,1223)
                      write( 6  ,1223)
1223                 format(/,' MAJOR ERROR IN QUADINIT: ',
     *                'EXPRO INDEX EXCEEDED LIMIT' ,
     *                ' (ii=1,5, 1223)',/)

                    endif
                   enddo      !do ii=1,5 (1220)
                  endif        !if(lpro ) 
c END LOOP TO ADD EXPRO  REACTION (1220)
c    CONTINUE AFTER BREAK LOOP
1221              continue


c END CONTROLS TO IDENTIFY EXCHANGE REACTIONS
                 endif    !if( (icx1.eq.ic1.and.icx2.eq.ic2) etc.
               endif    !if( (icy1.eq.icp1.and.icy2.eq.icp2) etc.

              enddo     !do nrp=1,c_nreac
c END LOOP - SEARCH THROUGH REACTIONS TO FIND MATCHING EXPRO REACTION (1300)


c END - CONTROLS TO SKIP AND CONTINUE LOOP
            endif        !if(c_icat(icy1).eq.1.or.c_icat(icy1).eq.4) 
           endif       !if(icx.ne.icy1.and.icx.ne.icy2.and.icy2.gt.0) 
          endif      !if(icy2.gt.0.and.icx2.eq.0) 

       enddo     !do nrx=1,c_nreac
c END   REACTION LOOP FOR EXCHANGE REACTIONS (1200)


c END EXCHANGE ANALYSIS.  
c ----------------------
c
c
c   IDENTIFY 'CROSS' REACTIONS FOR PAIRED SPECIES IN CASCADE
c NNRCPRO, NRCPRO, STOICPRO WITH SAME FORMAT AS NRCHEM, ABOVE.
c  THESE REPRESENT REACTIONS THAT PRODUCE THE SPECIES FROM 1/2 PAIR.

c NNRCPRX, NRCPRX, STOICPRX REPRESENT PRODUCTION REACTIONS ASSOCIATED 
c  WITH THE THIRD PAIR IN A TRIPLET - USED ONLY FOR O3-NO-NO2.
c
c JANUARY 2005:  THIS HAS BEEN DELETED.   IT RELATES TO OLD TWOSOLVE
c  AND NOXSOLVE - REPLACED BY NEW PAIR AND MULTISOLVE TREATMENTS.
c 
c 
c
c END CROSS-REACTION ANALYSIS.  END MECHANISM ANALYSIS.
c ----------------------

c  END QUADINIT
2000      return
          end


c --------------------------------------------------------------
c END quadinit.f
c
c ================================================================
